import acm.program.*;
import acm.util.*;

public class WaveTable extends ConsoleProgram {
	/* Frequencies of the C major scale. */
	private static final double[] FREQUENCIES = {
		261.626 * 2,  // C
		293.665 * 2,  // D
		329.628 * 2,  // E
		349.228 * 2,  // F
		391.995 * 2,  // G
		440.000 * 2,  // A
		493.883 * 2,  // B
		261.626 * 4,  // C
	};
	
	/**
	 * Generates a square wave with the given frequency and length.
	 * 
	 * @param frequency The frequency of the wave, in Hz.
	 * @param length The length of the sound clip, in seconds.
	 * @return Samples representing that square wave.
	 */
	private double[] squareWave(double frequency, double length) {
		/* Compute how many samples are needed - this is the sample rate
		 * (samples / sec) times the length (sec).
		 */
		int numSamples = (int)(StdAudio.SAMPLE_RATE * length);
		double[] soundSample = new double[numSamples];
		
		/* Compute the number of samples in one wave. */
		double wavelength = StdAudio.SAMPLE_RATE / frequency;
		
		for (int i = 0; i < soundSample.length; i++) {
			/* Determine where in the wave we are. */
			double position = i % wavelength;
			
			/* If this is the first half of the wave, set the intensity
			 * to +1; else set it to -1.
			 */
			if (position < wavelength / 2.0) {
				soundSample[i] = +1;
			} else {
				soundSample[i] = -1;
			}
		}
		
		return soundSample;
	}
	
	public void run() {
		setFont("DejaVuSerif-BOLD-24");

		/* Play a bunch of square waves to go through C Major. */
		for (int note = 0; note < FREQUENCIES.length; note++) {
			println("Frequency " + FREQUENCIES[note]);
			StdAudio.play(squareWave(FREQUENCIES[note], 0.25));
		}
	}
}
